"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unifyDependencyResolving = exports.throwErrIfHotModuleReplacementEnabled = exports.throwErrIfTargetNotSupported = exports.throwErrIfOutputPathNotSpecified = exports.forceEnableNodeGlobals = exports.forceDisableOutputtingEsm = exports.forceSetLibraryType = exports.forceDisableSplitChunks = void 0;
const lodash_1 = require("lodash");
const constants_1 = require("./constants");
const webpack_1 = require("./peers/webpack");
function forceDisableSplitChunks(compilerOptions) {
    (0, lodash_1.set)(compilerOptions, 'optimization.splitChunks', false);
}
exports.forceDisableSplitChunks = forceDisableSplitChunks;
function forceSetLibraryType(compilerOptions, libraryType) {
    (0, lodash_1.set)(compilerOptions, 'output.library.type', libraryType);
}
exports.forceSetLibraryType = forceSetLibraryType;
function forceDisableOutputtingEsm(compilerOptions) {
    (0, lodash_1.set)(compilerOptions, 'experiments.outputModule', false);
}
exports.forceDisableOutputtingEsm = forceDisableOutputtingEsm;
function forceEnableNodeGlobals(compilerOptions) {
    (0, lodash_1.set)(compilerOptions, 'node.__dirname', false);
    (0, lodash_1.set)(compilerOptions, 'node.__filename', false);
}
exports.forceEnableNodeGlobals = forceEnableNodeGlobals;
function throwErrIfOutputPathNotSpecified(compilerOptions) {
    const { output } = compilerOptions;
    if (!output.path)
        throw new Error(`The output.path in webpack config is not specified`);
}
exports.throwErrIfOutputPathNotSpecified = throwErrIfOutputPathNotSpecified;
function throwErrIfTargetNotSupported(compilerOptions) {
    const { target } = compilerOptions;
    if (!target)
        throw new Error(`The target in webpack config is not specified`);
    const targets = Array.isArray(target) ? target : [target];
    if (targets.some((t) => ['web', 'webworker'].includes(t))) {
        throw new Error(`The target '${target}' in webpack config is not supported ` +
            `when using plugin '${constants_1.pluginName}' (please use a node-compatible target)`);
    }
}
exports.throwErrIfTargetNotSupported = throwErrIfTargetNotSupported;
function throwErrIfHotModuleReplacementEnabled(compilerOptions) {
    const { plugins } = compilerOptions;
    for (const p of plugins) {
        if (p instanceof webpack_1.HotModuleReplacementPlugin ||
            p.constructor.name === 'HotModuleReplacementPlugin') {
            throw new Error(`Hot module replacement is not supported when using plugin '${constants_1.pluginName}'`);
        }
    }
}
exports.throwErrIfHotModuleReplacementEnabled = throwErrIfHotModuleReplacementEnabled;
function unifyDependencyResolving(compilerOptions, wantedType) {
    const { byDependency } = compilerOptions.resolve;
    if (!byDependency)
        return;
    if (!Object.prototype.hasOwnProperty.call(byDependency, wantedType))
        wantedType = 'unknown';
    const wantedOpts = byDependency[wantedType];
    for (const [type, opts] of Object.entries(byDependency)) {
        if (type !== wantedType) {
            Object.assign(opts, (0, lodash_1.pick)(wantedOpts, Object.keys(opts)));
        }
    }
}
exports.unifyDependencyResolving = unifyDependencyResolving;
//# sourceMappingURL=compilerOptions.js.map